from qmdl.mdl import Mdl
from qmdl.palette import palette
from PIL import Image, ImagePalette

class Helper:
    """
    This class contains a number of methods to perform common tasks on a Mdl.

    """
    def load(self, filename):
        self.mdl = Mdl()
        with open(filename, "rb") as file:
            self.mdl.read(file)
        self.filename = filename

    def save(self, filename=None):
        if filename is None:
            filename = self.filename
        with open(filename, "wb") as file:
            self.mdl.write(file)

    @staticmethod
    def rename_range(frames, start, name, length):
        """
        Sets sequencial names on a run of the given frames.

        Parameters:
        frames -- a list of frame objects
        start -- the index of the first frame to rename
        name -- a string given the root name to which numbers are appended
        length -- the number of frames to rename

        """
        for i in range(length):
            frames[i + start].name = (name + str(i + 1)).encode("ascii")

    def rename_frames(self, names):
        """
        Set sequencial names for the frames in the model.

        The names parameter needs a particular format. It should be a list, and
        each entry in the list should be a tuple:
        (name, length)
        name -- the root name of this frame sequence
        length -- the number of frames in this sequence

        This method always starts from frame 0, and renames each sequence
        starting from the frame one past the last renamed sequence. The
        intended use is to rename all the sequences in the model at once, so
        the names parameter should have entries that cover every frame.
        
        """
        i = 0
        for name in names:
            Helper.rename_range(self.mdl.frames, i, *(name))
            i += name[1]

    def group_frames(self, start, end, sequence_name, duration=0.1):
        """
        Takes the frames between start and end and makes them a framegroup.

        Parameters:
        start -- the index of the first frame to include in the framegroup
        end -- the index of the last frame to include in the framegroup
        sequence_name -- what to call the new framegroup
        duration -- how long each frame in the framegroup will last

        Note that this function modifies the frames list - the new framegroup
        replaces the frames that previously sat between start and end. Bear
        this in mind when calling the function multiple times in a row, all the
        frames after the first framegroup will have moved down the list and
        have new indices. One way to make this simpler is to start with the
        final framegroup in your model and work your way to the first one. This
        way you've already finished with the frames which move positions.
        
        """
        # python expects end to be the frame 1 past the frame we want
        end = end + 1
        head = self.mdl.frames[:start]
        ungrouped_frames = self.mdl.frames[start:end]
        tail = self.mdl.frames[end:]
        frame_group = Mdl.FrameGroup()
        frame_group.name = sequence_name.encode("ascii")
        frame_group.duration = [(1 + x) * duration for x in range(end - start)]
        frame_group.frames = ungrouped_frames
        head.append(frame_group)
        head.extend(tail)
        self.mdl.frames = head
        
    def append_skin(self, filename):
        """
        Import an image from the given filename and append it as a skin.

        This method uses the Python Image Library, and will open any file it
        supports. If the loaded image uses an indexed palette, the function
        assumes it is in the Quake palette and performs no conversion. If the
        file is in RGB format it will be dithered to the Quake palette. There
        is no guarantee that the conversion will be pretty, it is advised to
        perform conversion externally for best results.
        
        """
        im=Image.open(filename)
        if im.mode == "RGB":
            pim=Image.new("P", (self.mdl.skinwidth, self.mdl.skinheight))
            pim.putpalette(palette)
            pim=im.quantize(palette=pim)
        else:
            pim = im
        sk=Mdl.Skin(self.mdl.skinwidth, self.mdl.skinheight)
        sk.pixels=pim.tostring()
        self.mdl.skins.append(sk)
